/*
	deck.c
	Copyright (C) 2001 David Fifield

	This file is part of solitairecrypt.

	solitairecrypt is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2, or (at
	your option) any later version.

	solitairecrypt is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "deck.h"

deck *deck_new(void)
{
	deck *d;

	d = (deck *) malloc(sizeof(deck));
	if (d == NULL)
		return NULL;
	deck_init(d);

	return d;
}

void deck_init(deck *d)
{
	unsigned int i;

	d->top = 0;
	d->cards[0].prev = DECK_SIZE - 1;
	d->cards[DECK_SIZE - 1].next = 0;
	for (i = 1; i < DECK_SIZE; i++)
		d->cards[i].prev = i - 1;
	for (i = 0; i < DECK_SIZE - 1; i++)
		d->cards[i].next = i + 1;

	return;
}

void deck_free(deck *d)
{
	free(d);

	return;
}

void deck_move_jokers(deck *d)
{
	unsigned int i;

	i = d->cards[JOKER_A].next;
	if (d->top == JOKER_A)
		d->top = i;
	d->cards[i].prev = d->cards[JOKER_A].prev;
	d->cards[d->cards[JOKER_A].prev].next = i;
	d->cards[JOKER_A].next = d->cards[i].next;
	d->cards[d->cards[i].next].prev = JOKER_A;
	d->cards[JOKER_A].prev = i;
	d->cards[i].next = JOKER_A;

	i = d->cards[JOKER_B].next;
	if (d->top == JOKER_B)
		d->top = i;
	d->cards[i].prev = d->cards[JOKER_B].prev;
	d->cards[d->cards[JOKER_B].prev].next = i;
	i = d->cards[i].next;
	d->cards[JOKER_B].next = d->cards[i].next;
	d->cards[d->cards[i].next].prev = JOKER_B;
	d->cards[JOKER_B].prev = i;
	d->cards[i].next = JOKER_B;

	return;
}

void deck_triple_cut(deck *d)
{
	unsigned int a, b;
	unsigned int i;

	for (a = d->top; a < JOKER_A; a = d->cards[a].next)
		;
	if (a == JOKER_A)
		b = JOKER_B;
	else
		b = JOKER_A;
	b = d->cards[b].next;
	if (a == d->top) {
		d->top = b;
	} else if (b == d->top) {
		d->top = a;
	} else {
		i = d->cards[d->top].prev;
		d->cards[d->cards[d->top].prev].next = a;
		d->cards[d->cards[a].prev].next = b;
		d->cards[d->cards[b].prev].next = d->top;
		d->cards[d->top].prev = d->cards[b].prev;
		d->cards[b].prev = d->cards[a].prev;
		d->cards[a].prev = i;
		d->top = b;
	}

	return;
}

void deck_count_cut(deck *d)
{
	unsigned int count;

	count = d->cards[d->top].prev + 1;
	if (count > DECK_SIZE - 1)
		count = DECK_SIZE - 1;
	deck_count_cut_num(d, count);

	return;
}

void deck_count_cut_num(deck *d, unsigned int n)
{
	unsigned int i;

	n %= DECK_SIZE;
	if (n == DECK_SIZE - 1)
		return;
	i = d->top;
	if (n < DECK_SIZE / 2) {
		for ( ; n > 0; n--)
			i = d->cards[i].next;
	} else {
		for ( ; n < DECK_SIZE; n++)
			i = d->cards[i].prev;
	}
	n = d->cards[d->top].prev;
	d->cards[d->cards[d->top].prev].next = i;
	d->cards[d->cards[i].prev].next = n;
	d->cards[d->cards[n].prev].next = d->top;
	d->cards[d->top].prev = d->cards[n].prev;
	d->cards[n].prev = d->cards[i].prev;
	d->cards[i].prev = n;
	d->top = i;

	return;
}

void deck_print(deck *d)
{
	unsigned int count;
	unsigned int i;

	i = d->top;
	count = 0;
	do {
		switch (i) {
			case JOKER_A:
				printf(" A");
				break;
			case JOKER_B:
				printf(" B");
				break;
			default:
				printf("%2d", i + 1);
				break;
		}
		i = d->cards[i].next;
		if (count == 26 || count == 53)
			putchar('\n');
		else
			putchar(' ');
		count++;
	} while (i != d->top);

	return;
}

